@@ -1,68 +1,13 @@
+import logging
+
 from suds import WebFault
 from django.conf import settings
 
-from .client import Auth, Document, Property, PropertyGroup, Repository
-from .facades import Category, Keyword, DirectoryListing
-from .models import OpenKmFolderList, OpenKmDocument
+from django_openkm import utils
+from .client import PropertyGroup, Repository
+from .facades import Category, Keyword, DirectoryListing, Property
 from .utils import find_key
 
-class SyncPropertiesException(Exception):
-    pass
-
-class SyncProperties(object):
-
-    PROPERTY_GROUP = 'okg:customProperties'
-
-    # django model attributes -> openkm properties
-    MODEL_PROPERTY_MAP = {
-        'name': 'Title',
-        'description': 'Description',
-        }
-
-    def __init__(self):
-        auth = Auth()
-        auth.login()
-        self.property_group = PropertyGroup()
-
-    def document_has_property_group(self, document, group_name):
-        return self.property_group.has_group(document.path, group_name)
-
-    def update_properties(self, resource, properties):
-        for property in properties.item:
-            if property.label in self.MODEL_PROPERTY_MAP.values():
-                django_attr = find_key(self.MODEL_PROPERTY_MAP, property.label)
-                property.value = resource.__getattribute__(django_attr)
-        return properties
-
-    def update_django_model_attributes(self, model, properties):
-        for property in properties.item:
-            if property.label in self.MODEL_PROPERTY_MAP.values():
-                model_attribute_name = find_key(self.MODEL_PROPERTY_MAP, property.label)
-                setattr(model, model_attribute_name, property.value)
-        return model.save()
-
-    def django_to_openkm(self, resource, document):
-        # if document has the group, then grab the properties and remove it, as
-        # we need to re-add it if we want to modify it
-        if self.document_has_property_group(document, self.PROPERTY_GROUP):
-            properties = self.property_group.get_properties(document.path, self.PROPERTY_GROUP)
-            self.property_group.remove_group(document.path, self.PROPERTY_GROUP)
-
-        self.property_group.add_group(document.path, self.PROPERTY_GROUP)
-
-        if not properties:
-            properties = self.property_group.get_properties(document.path, self.PROPERTY_GROUP)
-
-        updated_properties = self.update_properties(resource, properties)
-        self.property_group.set_properties(document.path, self.PROPERTY_GROUP, updated_properties)
-
-    def openkm_to_django(self, resource, document):
-        if not self.document_has_property_group(document, self.PROPERTY_GROUP):
-            raise SyncPropertiesException('OpenKM document does not have %s' % self.PROPERTY_GROUP)
-        properties = self.property_group.get_properties(document.path, self.PROPERTY_GROUP)
-
-
-
 class SyncKeywords(object):
 
     def __init__(self):
@@ -119,6 +64,7 @@ class SyncKeywords(object):
         tags = ','.join(keywords)
         document.update_tags(tags)
 
+
 class SyncCategories(object):
     """
     Syncronises categories from Django to OpenKM.
@@ -136,14 +82,15 @@ class SyncCategories(object):
         """
         if not parent_path:
             parent_path = self.category.get_category_root().path
+        logging.info("parent path: %s", parent_path)
 
         for category_name in self.MODEL_CATEGORY_MAP.values():
             path = self.category.construct_valid_path_string(parent_path, category_name)
             try:
                 self.category.create(path)
-                logger.info("%s created" % path)
+                logging.info("%s created" % path)
             except:
-                logger.info("%s creation failed" % path)
+                logging.info("%s creation failed" % path)
 
     def get_child_categories(self, parent_path):
         """
@@ -165,6 +112,7 @@ class SyncCategories(object):
         The parent category should already exist.
         :param model_class: model class object
         :param category_name: string (name of the Category on OpenKM)
+        @todo confirm the replace char
         """
         objects = model_class.objects.all()
 
@@ -173,12 +121,14 @@ class SyncCategories(object):
                 self.category.create('/okm:categories/%s/%s' % (category_name,\
                                                                 object.__unicode__().replace('/','')))
             except:
-                logger.info("%s creation failed" % object.__unicode__().replace('/',''))
+                logging.info("%s creation failed" % object.__unicode__().replace('/',''))
 
-    def openkm_to_django(self):
+    def openkm_to_django(self, resource):
+        from django_openkm import facades
+        #facades.Category
         pass
 
-    def get_many_to_many_fields_from_model(self, document, related_model_class):
+    def get_objects_from_m2m_model(self, document, related_model_class):
         """
         Accepts a Document and a single class object of a many-to-many field
         :returns queryset the related model objects associated with the given resource
@@ -186,6 +136,8 @@ class SyncCategories(object):
         method_name = '%s_set' % related_model_class.__name__.lower()
         _set = getattr(document, method_name)
 
+#        import ipdb; ipdb.set_trace()
+
         return _set.all()
 
     def get_related_objects_from_model(self, document, related_model_name):
@@ -204,70 +156,90 @@ class SyncCategories(object):
             _member = getattr(document, method_name)
             return _member.all()
         else:
-            logger.error("Object does not have method region_set")
+            logging.error("Object does not have method %s" % method_name)
 
 
-class SyncResourceException(Exception):
-    pass
+class SyncProperties(object):
+
+    def __init__(self):
+        self.property = Property()
+        self.property_group = PropertyGroup()
+
+    def django_to_openkm(self, resource):
+        self.PROPERTY_GROUP_MAP = self.populate_property_group_map(resource)
+        logging.info(self.PROPERTY_GROUP_MAP)
+
+        for property_group in self.PROPERTY_GROUP_MAP:
+            logging.debug("property_group: %s", property_group)
+            if not self.property_group.has_group(resource.okm_path, property_group):
+                self.property_group.add_group(resource.okm_path, property_group)
+
+            properties = self.property_group.get_properties(resource.okm_path, property_group)
+
+            # update the properties values and set them on OpenKM
+            updated_properties = self.property.update_document_properties(properties, self.PROPERTY_GROUP_MAP[property_group])
+            self.property_group.add_group(resource.okm_path, property_group)
+            self.property_group.set_properties(resource.okm_path, property_group, updated_properties)
+
+    def openkm_to_django(self, resource):
+        self.PROPERTY_GROUP_MAP = self.reverse_mapping()
+        document_property_groups = self.property.get_property_groups_for_document(resource.okm_path)
+
+        for property_group in document_property_groups[0]:
+            document_properties = self.property.get_document_properties_for_group(resource.okm_path, property_group.name)
+            property_map = self.PROPERTY_GROUP_MAP[property_group.name]
+            self.set_attributes(property_map, document_properties[0], resource)
+
+    def set_attributes(self, property_map, document_properties, resource):
+        for document_property in document_properties:
+            if property_map.get(document_property.name, None):
+                # the property exists in the map
+                name = property_map.get(document_property.name, None)
+                if not isinstance(name, tuple):
+                    # this is a normal text attribute so save it
+                    setattr(resource, name, document_property.value)
+                else:
+                    # this is a Django 'choice' so lookup if required
+                    name, choices = name
+                    option = self.get_option(document_property.options)
+                    if choices:
+                        value = utils.find_key(dict(choices), option)
+                        setattr(resource, name, value)
+                    else:
+                        setattr(resource, name, option)
+        resource.save()
+
+    def get_option(self, options):
+        for option in options:
+            if option.selected:
+                return option.label
 
-class SyncResource(object):
     """
-    Syncs a Resource object to OpenKM
+    @todo The two functions below violate the DRY principle and need to be merged into a single dictionary
     """
-    def __init__(self, document):
-        if not isinstance(document, OpenKmDocument):
-            raise SyncResourceException('Must pass a Resource instance to be synced')
-        self.auth = Auth()
-        self.auth.login()
-        self.document = Document()
-        self.repository = Repository()
-        self.sync_keywords = SyncKeywords()
-        self.sync_categories = SyncCategories()
+    def populate_property_group_map(self, resource):
+        return {
+            "okg:customProperties": {
+                "okp:customProperties.title": resource.name,
+                'okp:customProperties.description': resource.description,
+                'okp:customProperties.languages': resource.language,
+                },
+            "okg:salesProperties": {
+                'okp:salesProperties.assetType': resource.get_type_display(),
+                }
+        }
 
-    def new_openkm_document_object(self):
-        """ Returns a local object instance """
-        return self.document.new()
-
-    def document_exists_on_openkm(self, path):
-        return self.repository.has_node(path)
-
-    def build_openkm_path(self, base_path, file_name):
-        return "%s%s" % (base_path, file_name)
-
-    def upload_to_openkm(self, file):
-        """
-        Uploads the document to the OpenKM server
-        :param file: a file object
-        """
-        path = self.build_openkm_path(settings.OPENKM['UploadRoot'], file.name)
-        openkm_document = self.new_openkm_document_object()
-        openkm_document.path = settings.OPENKM['UploadRoot'] + file.name
-        content = utils.make_file_java_byte_array_compatible(file)
-
-        try:
-            return self.document.create(openkm_document, content)
-        except Exception, e:
-            logger.error("There was a problem uploading the document: %s" % file.name)
-
-    def keywords(self):
-        """
-        TAGS -> KEYWORDS
-        Writes the tags to OpenKM as keywords and confirms that they have been added
-        :returns boolean:  True on success, False on fail
-        """
-        tags = self.sync_keywords.get_tags_from_resource(self.document)
-        self.sync_keywords.write_keywords_to_openkm_document(self.path, tags)
-        return self.sync_keywords.confirm_keywords_written_to_openkm(self.path, tags)
-
-    def categories(self):
-        for related_model_class in self.sync_categories.MODEL_CATEGORY_MAP.keys():
-            self.sync_categories.get_many_to_many_fields_from_model(self.document, related_model_class)
-
-    def properties(self):
-        pass
-
-    def file(self):
-        pass
+    def reverse_mapping(self):
+        return {
+            "okg:customProperties": {
+                "okp:customProperties.title": 'name',
+                'okp:customProperties.description': 'description',
+                'okp:customProperties.languages': ('language', None)
+                },
+            "okg:salesProperties": {
+                'okp:salesProperties.assetType': ('type', RESOURCE_TYPES),
+                }
+        }
 
 
 class SyncFolderList(object):
@@ -276,15 +248,26 @@ class SyncFolderList(object):
     @todo make this generic to pickup all the folders across OpenKM, not just
     categories
     """
+    DIRECTORY_SEPARATOR = "/"
+
     def __init__(self):
         self.category = Category()
         self.dir = DirectoryListing()
         self.repository = Repository()
 
-    def execute(self):
+    def execute(self, klass):
+        """
+        :param klass your django model class storing the OpenKM folder list
+        """
+        logging.info('Class: %s', klass)
+
         paths = self.get_list_of_root_paths()
+        logging.info(paths)
+
         folders = self.traverse_folders(paths)
-        self.save(folders)
+        logging.info(folders)
+
+        self.save(folders, klass)
 
     def get_list_of_root_paths(self):
         return [self.category.get_category_root().path, self.repository.get_root_folder().path]
@@ -296,16 +279,18 @@ class SyncFolderList(object):
 
         return folders
 
-    def save(self, folders):
+    def save(self, folders, klass):
         for folder in folders:
+            logging.info(folder)
             try:
-                cl, created = OpenKmFolderList.objects.get_or_create(uuid=folder.uuid)
-                cl.author = folder.author
-                cl.created = folder.created
-                cl.has_childs = folder.hasChilds
-                cl.path = folder.path
-                cl.permissions = folder.permissions
-                cl.subscribed = folder.subscribed
+                cl, created = klass.objects.get_or_create(okm_uuid=folder.uuid)
+                cl.okm_author = folder.author
+                cl.okm_created = folder.created
+                cl.okm_has_childs = folder.hasChilds
+                cl.okm_path = folder.path
+                cl.okm_permissions = folder.permissions
+                cl.okm_subscribed = folder.subscribed
                 cl.save()
             except Exception, e:
-                logger.error(e)
+                print e
+
