@@ -1,21 +1,10 @@
 import base64
-import logging
 
 import suds
 
 """
 Some useful helper and decorator functions
 """
-
-def try_except(fn):
-    """ A decorator to catch suds exceptions """
-    def wrapped(*args, **kwargs):
-        try:
-            fn(*args, **kwargs)
-        except suds.WebFault as detail:
-            return detail
-    return wrapped
-
 def make_file_java_byte_array_compatible(file_obj):
     """ 
     Reads in a file and converts it to a format accepted as Java byte array 
@@ -46,7 +35,17 @@ def find_key(dic, val):
     """return the key of dictionary dic given the value"""
     return [k for k, v in dic.iteritems() if v == val][0]
 
-logger = logging.getLogger('tce')
-logger.setLevel(logging.DEBUG)
-formatter = logging.Formatter('%(levelname)-8s %(message)s')
+def get_category_from_path(path):
+    """
+    Document.get_properties returns Category path and uuid information, so doing this
+    saves us making a second webservice call
+    :param path: string
+    :return list
+    """
+    parts = path.split('/')
+    if 'categories' not in parts[1]:
+        return []
+    else:
+        return parts[2:]
+
 
